function ArkInventory.MenuMainOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local loc_id
		if frame.ARK_Data then
			loc_id = frame.ARK_Data.loc_id
		end
		
		local cp = ArkInventory.LocationPlayerInfoGet( loc_id )
	
		local anchorpoints = {
			[ArkInventory.Const.Anchor.TopRight] = ArkInventory.Localise["GENERIC_TOPRIGHT"],
			[ArkInventory.Const.Anchor.BottomRight] = ArkInventory.Localise["GENERIC_BOTTOMRIGHT"],
			[ArkInventory.Const.Anchor.BottomLeft] = ArkInventory.Localise["GENERIC_BOTTOMLEFT"],
			[ArkInventory.Const.Anchor.TopLeft] = ArkInventory.Localise["GENERIC_TOPLEFT"],
		}

		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "TOPLEFT"
		else
			p = "TOPLEFT"
			rp = "TOPRIGHT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
				
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Const.Program.Name,
						"isTitle", true,
						"textHeight", 14
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Global.Version,
						"notClickable", true
					)

					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["OPT_CONFIG"],
						"closeWhenClicked", true,
						"func", function( )
							ArkInventory.Lib.Dialog:Open( ArkInventory.Const.Frame.Options.Internal )
						end
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_MAIN_ACTIONS"],
						"hasArrow", true,
						"value", "ACTIONS"
					)
					
				end
				
				
				if level == 2 and value then
				
					if value == "ACTIONS" then
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_MAIN_ACTIONS"],
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_MAIN_RELOAD"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_RELOAD"],
							"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_RELOAD_TEXT"],
							--"icon", ArkInventory.Const.Texture.Action.Refresh,
							"closeWhenClicked", true,
							"func", function( )
								ArkInventory.ItemCacheClear( )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
							end
						)
						
					end
					
					
				end
				
				
				ArkInventory.Lib.DewDrop:AddLine( )
				ArkInventory.Lib.DewDrop:AddLine(
					"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
					"closeWhenClicked", true
				)
				
			end
			
		)
	
	end
	
end

function ArkInventory.MenuBarOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then

		ArkInventory.Lib.DewDrop:Close( )
		
	else

		local loc_id = frame.ARK_Data.loc_id
		local bar_id = frame.ARK_Data.bar_id
		local bar_name = ArkInventory.LocationOptionGet( loc_id, { "bar", "name", "label", bar_id } ) or ""
		local cp = ArkInventory.LocationPlayerInfoGet( loc_id )
		
		
		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "TOPLEFT"
		else
			p = "TOPLEFT"
			rp = "TOPRIGHT"
		end
	
		local category = {
			["type"] = { "SYSTEM", "CONSUMABLE", "TRADE_GOODS", "SKILL", "CLASS", "EMPTY", "CUSTOM", "RULE", },
		}
		
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
			
				if level == 1 then
				
					ArkInventory.Lib.DewDrop:AddLine(
						"text", string.format( ArkInventory.Localise["MOD_MENU_BAR_INFORMATION_TITLE"], bar_id ),
						"isTitle", true,
						"textHeight", 12
					)

					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_CURRENT"] .. ":",
						"isTitle", true
					)
					
					local has_entries = false
					for _, v in ipairs( category.type ) do
						if ArkInventory.Category_Bar_HasEntries( loc_id, bar_id, v ) then
							has_entries = true
							ArkInventory.Lib.DewDrop:AddLine(
								"text", ArkInventory.Localise["CATEGORY_" .. v],
								"hasArrow", true,
								"value", "CATEGORY_CURRENT_" .. v
							)
						end
					end
					
					if not has_entries then
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_NONE"],
							"disabled", true
						)
					end

					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_ASSIGN"] .. ":",
						"isTitle", true
					)
					for _, v in ipairs( category.type ) do
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["CATEGORY_" .. v],
							"hasArrow", true,
							"value", "CATEGORY_ASSIGN_" .. v
						)
					end
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_BAR_OPTIONS"],
						"hasArrow", true,
						"value", "BAR_OPTIONS"
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
				
				if level == 2 and value then

					if strsub( value, 1, 17 ) == "CATEGORY_CURRENT_" then

						local cat_type = strmatch( value, "^CATEGORY_CURRENT_(.+)" )
				
						if cat_type ~= nil then
					
							ArkInventory.Lib.DewDrop:AddLine(
								"text", ArkInventory.Localise["CATEGORY_" .. cat_type],
								"isTitle", true,
								"textHeight", 12
							)

							ArkInventory.Lib.DewDrop:AddLine( )
							local has_entries = false
							for _, cat in ArkInventory.spairs( ArkInventory.Global.Category, function(a,b) return ArkInventory.Global.Category[a].fullname < ArkInventory.Global.Category[b].fullname end ) do

								local t = cat.type_code
								local cat_bar, def_bar = ArkInventory.Category_Location_Get( loc_id, cat.id )
								
								if abs( cat_bar ) == bar_id and not def_bar then
							
									if t == "RULE" then
										local _, cat_code = ArkInventory.CategoryCodeSplit( cat.id )
										if not ArkInventory.db.profile.option.rule[cat_code] then
											t = "DO_NOT_USE" -- don't display disabled rules
										end
									end
						
									if cat_type == t then
		
										local tag_colour = GREEN_FONT_COLOR_CODE
										if cat_bar < 0 then
											tag_colour = RED_FONT_COLOR_CODE
										end
		
										has_entries = true
										ArkInventory.Lib.DewDrop:AddLine(
											"text", tag_colour .. cat.name,
											"tooltipTitle", cat.fullname,
											"hasArrow", true,
											"value", "CATEGORY_CURRENT_OPTION_" .. cat.id
										)
										
									end
						
								end
	
							end
							
							if not has_entries then
								ArkInventory.Lib.DewDrop:AddLine(
									"text", ArkInventory.Localise["GENERIC_NONE"],
									"disabled", true
								)
							end
							
						end

					end

					
					if strsub( value, 1, 16 ) == "CATEGORY_ASSIGN_" then
					
						local cat_type = strmatch( value, "^CATEGORY_ASSIGN_(.+)" )
				
						if cat_type ~= nil then
					
							ArkInventory.Lib.DewDrop:AddLine(
								"text", ArkInventory.Localise["CATEGORY_" .. cat_type],
								"isTitle", true,
								"textHeight", 12
							)

							ArkInventory.Lib.DewDrop:AddLine( )
							for _, cat in ArkInventory.spairs( ArkInventory.Global.Category, function(a,b) return ArkInventory.Global.Category[a].fullname < ArkInventory.Global.Category[b].fullname end ) do

								local t = cat.type_code
								local cat_bar, def_bar = ArkInventory.Category_Location_Get( loc_id, cat.id )
							
								if abs( cat_bar ) == bar_id and not def_bar then
									t = "DO_NOT_USE" -- change the category so it doesn't display
								end

								if t == "RULE" then
									local _, cat_code = ArkInventory.CategoryCodeSplit( cat.id )
									if not ArkInventory.db.profile.option.rule[cat_code] then
										t = "DO_NOT_USE" -- don't display disabled rules
									end
								end
								
								if cat_type == t then
	
									local n = cat.name
	
									if not def_bar then
										-- category is assigned to a bar - show the user where
										n = LIGHTYELLOW_FONT_COLOR_CODE .. cat.name .. GREEN_FONT_COLOR_CODE .. "  [" .. ArkInventory.Category_Location_Get( loc_id, cat.id ) .. "]" .. FONT_COLOR_CODE_CLOSE
									end
	
									ArkInventory.Lib.DewDrop:AddLine(
										"text", n,
										"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAR_CATEGORY"],
										"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_TEXT"], cat.fullname ),
										"func", function( )
											ArkInventory.Category_Location_Set( loc_id, cat.id, bar_id )
											ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
										end
									)
								end
	
							end
							
						end
						
					end

					
					if value == "BAR_OPTIONS" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_BAR_OPTIONS"],
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", string.format( "%s: %s", ArkInventory.Localise["MOD_MENU_MAIN_BARS_NAME"], bar_name ),
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_NAME"],
							--"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_BARS_NAME_TEXT"],
							"hasArrow", true,
							"hasEditBox", true,
							"editBoxText", bar_name,
							"editBoxFunc", function( v )
								bar_name = string.trim( v )
								ArkInventory.LocationOptionSet( loc_id, { "bar", "name", "label", bar_id }, bar_name )
								ArkInventory.Frame_Bar_Paint_All( )
								ArkInventory.Lib.DewDrop:Close( )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_BAR_CLEAR"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAR_CLEAR"],
							"tooltipText", ArkInventory.Localise["MOD_MENU_BAR_CLEAR_TEXT"],
							"closeWhenClicked", true,
							"func", function( )
								ArkInventory.Category_Bar_Clear( loc_id, bar_id )
								ArkInventory.Frame_Bar_Label_Clear( loc_id, bar_id )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
							end
						)
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_BAR_INSERT"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAR_INSERT"],
							"tooltipText", ArkInventory.Localise["MOD_MENU_BAR_INSERT_TEXT"],
							"closeWhenClicked", true,
							"func", function( )
								ArkInventory.Category_Bar_Insert( loc_id, bar_id )
								ArkInventory.Frame_Bar_Label_Insert( loc_id, bar_id )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
							end
						)
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_BAR_REMOVE"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAR_REMOVE"],
							"tooltipText", ArkInventory.Localise["MOD_MENU_BAR_REMOVE_TEXT"],
							"closeWhenClicked", true,
							"func", function( )
								ArkInventory.Category_Bar_Remove( loc_id, bar_id )
								ArkInventory.Frame_Bar_Label_Remove( loc_id, bar_id )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_BAR_MOVE_UP"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAR_MOVE_UP"],
							"tooltipText", ArkInventory.Localise["MOD_MENU_BAR_MOVE_UP_TEXT"],
							"closeWhenClicked", true,
							"func", function( )
								ArkInventory.Category_Bar_Up( loc_id, bar_id )
								ArkInventory.Frame_Bar_Label_Up( loc_id, bar_id )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_BAR_MOVE_DOWN"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAR_MOVE_DOWN"],
							"tooltipText", ArkInventory.Localise["MOD_MENU_BAR_MOVE_DOWN_TEXT"],
							"disabled", bar_id < 2,
							"closeWhenClicked", true,
							"func", function( )
								ArkInventory.Category_Bar_Down( loc_id, bar_id )
								ArkInventory.Frame_Bar_Label_Down( loc_id, bar_id )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
							end
						)
					
					end
					
				end

				
				if level == 3 and value then
				
					if strsub( value, 1, 24 ) == "CATEGORY_CURRENT_OPTION_" then
					
						local cat_id = strmatch( value, "^CATEGORY_CURRENT_OPTION_(.+)" )
				
						if cat_id ~= nil then
					
							local cat = ArkInventory.Global.Category[cat_id]
						
							ArkInventory.Lib.DewDrop:AddLine(
								"text", cat.fullname,
								"isTitle", true,
								"textHeight", 12
							)

							ArkInventory.Lib.DewDrop:AddLine( )
							
							local cv = ArkInventory.Category_Location_Get( loc_id, cat_id )
							local ch = false
							
							if cv < 0 then
								ch = true
								cv = abs( cv )
							end
							
							ArkInventory.Lib.DewDrop:AddLine(
								"text", string.format( ArkInventory.Localise["MOD_MENU_ITEM_MOVE"], LIGHTYELLOW_FONT_COLOR_CODE .. cat.fullname .. FONT_COLOR_CODE_CLOSE ),
								"hasArrow", true,
								"hasSlider", true,
								"sliderValue", cv,
								"sliderMin", 1,
								"sliderMax", ArkInventory.Global.Location[loc_id].Layout.bar_count + 1,
								"sliderStep", 1,
								"sliderFunc", function( nv )
									if cv ~= nv then
										if ch then
											nv = 0 - nv
										end
										ArkInventory.Category_Location_Set( loc_id, cat_id, nv )
										ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
									end
								end
							)

							ArkInventory.Lib.DewDrop:AddLine( )
							ArkInventory.Lib.DewDrop:AddLine(
								"text", ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_REMOVE"],
								"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_REMOVE"],
								"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_REMOVE_TEXT"], cat.fullname, bar_id ),
								"func", function( )
									ArkInventory.Category_Location_Set( loc_id, cat_id, nil )
									ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
								end
							)

							ArkInventory.Lib.DewDrop:AddLine(
								"text", ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_HIDDEN"],
								"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_HIDDEN"],
								"tooltipText", ArkInventory.Localise["MOD_MENU_BAR_CATEGORY_HIDDEN_TEXT"],
								"checked", ArkInventory.Category_Hidden_Check( loc_id, cat_id ),
								"func", function( )
									ArkInventory.Category_Hidden_Toggle( loc_id, cat_id )
									ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
								end
							)
					
						end
						
					end

				
				end
				
			end
		)

	end
	
end

function ArkInventory.MenuItemOpen( frame )

	if ArkInventory.Global.Mode.Edit == false then
		return
	end

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
		
	else

		local loc_id = frame.ARK_Data.loc_id
		local bag_id = frame.ARK_Data.bag_id
		local bliz_id = ArkInventory.BagID_Blizzard( loc_id, bag_id )
		local slot_id = frame.ARK_Data.slot_id
		local cp = ArkInventory.LocationPlayerInfoGet( loc_id )
		local i = ArkInventory.Frame_Item_GetDB( frame )
		
		local isEmpty = false
		if not i or i.h == nil then
			isEmpty = true
		end
	
	
		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "TOPLEFT"
		else
			p = "TOPLEFT"
			rp = "TOPRIGHT"
		end
	
		local ic = select( 4, GetItemQualityColor( i.q ) )
		local cat0, cat1, cat2 = ArkInventory.ItemCategoryGet( i )
		local bar_id = abs( ArkInventory.Category_Location_Get( loc_id, cat0 ) )
		
		local categories = { "SYSTEM", "CONSUMABLE", "TRADE_GOODS", "SKILL", "CLASS", "EMPTY", "CUSTOM", }
		
		cat0 = ArkInventory.Global.Category[cat0]
		
		if cat1 then
			cat1 = ArkInventory.Global.Category[cat1]
		end
		
		cat2 = ArkInventory.Global.Category[cat2]
		
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
			
				if level == 1 then

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_ITEM_INFORMATION_TITLE"] .. ":",
						"isTitle", true,
						"textHeight", 12
					)
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_ITEM_ITEM"] .. ": " .. ic .. ( select( 3, ArkInventory.ObjectInfo( i.h ) ) or "" )
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					if cat1 then
					
						-- item has a category, that means it's been specifically assigned away from the default
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_ITEM_CURRENT_ASSIGNMENT"] .. ": " .. GREEN_FONT_COLOR_CODE .. cat1.fullname .. FONT_COLOR_CODE_CLOSE,
							"notClickable", true
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_ITEM_DEFAULT_ASSIGNMENT"] .. ": " .. LIGHTYELLOW_FONT_COLOR_CODE .. cat2.fullname .. FONT_COLOR_CODE_CLOSE,
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_ITEM_DEFAULT_RESET"],
							"tooltipText", ArkInventory.Localise["MOD_MENU_ITEM_DEFAULT_RESET_TEXT"],
							"closeWhenClicked", true,
							"func", function( )
								ArkInventory.ItemCategorySet( i, nil )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
							end
						)
					
					else
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_ITEM_DEFAULT_ASSIGNMENT"] .. ": " .. LIGHTYELLOW_FONT_COLOR_CODE .. cat2.fullname .. FONT_COLOR_CODE_CLOSE,
							"notClickable", true
						)
					
					end
					
					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_ITEM_ASSIGN_CHOICES"] .. ":",
						"isTitle", true
					)
					for _, v in ipairs( categories ) do
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["CATEGORY_" .. v],
							"disabled", isEmpty,
							"hasArrow", true,
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_ITEM_ASSIGN_THIS"],
							"tooltipText", ArkInventory.Localise["MOD_MENU_ITEM_ASSIGN_THIS_TEXT"],
							"value", "CATEGORY_ASSIGN_" .. v
						)
					end
					
					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", string.format( ArkInventory.Localise["MOD_MENU_ITEM_MOVE"], LIGHTYELLOW_FONT_COLOR_CODE .. cat0.fullname .. FONT_COLOR_CODE_CLOSE ),
						"hasArrow", true,
						"hasSlider", true,
						"sliderValue", ArkInventory.Category_Location_Get( loc_id, cat0.id ),
						"sliderMin", 1,
						"sliderMax", ArkInventory.Global.Location[loc_id].Layout.bar_count + 1,
						"sliderStep", 1,
						"sliderFunc", function( v )
							if ArkInventory.Category_Location_Get( loc_id, cat0.id ) ~= v then
								ArkInventory.Category_Location_Set( loc_id, cat0.id, v )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
							end
						end
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG"],
						"hasArrow", true,
						"value", "DEBUG_INFO"
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
				
				if level == 2 and value then
				
--[[
					if value == "ITEM_RULE_ADD" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", string.upper( ArkInventory.Localise["MOD_MENU_ITEM_RULE_ADD"] ),
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						local n
						for _, v in ArkInventory.spairs( ArkInventory.Global.Category, function(a,b) return ArkInventory.Global.Category[a].order < ArkInventory.Global.Category[b].order end ) do
							if v.type_code == "RULE" and ArkInventory.db.profile.option.rule[v.id] and ArkInventory.db.global.option.category[ArkInventory.Const.Category.Type.Rule].data[v.id].compiled and not ArkInventory.db.global.option.category[ArkInventory.Const.Category.Type.Rule].data[v.id].damaged then
								
								local checked = ArkInventory.RuleAppliesToItem( v.id, i )
								
								if checked then
									n = GREEN_FONT_COLOR_CODE .. v.name .. FONT_COLOR_CODE_CLOSE
								else
									n = v.name
								end
								
								ArkInventory.Lib.DewDrop:AddLine(
									"text", n,
									"closeWhenClicked", true,
									"checked", checked,
									"func", function( )
										ArkInventory.RuleItemAdd( v.id, i.h )
									end
								)
							end
						
						end
						
					end
]]--
				
					if value == "DEBUG_INFO" then
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG"],
							"isTitle", true,
							"textHeight", 12
						)
						
						local class, id = ArkInventory.ObjectStringDecode( i.h )
						local bagtype = ArkInventory.Const.Slot.Data[ArkInventory.BagType( bliz_id )].type
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s (%s)", ArkInventory.Localise["GENERIC_LOCATION"], LIGHTYELLOW_FONT_COLOR_CODE, loc_id, ArkInventory.Global.Location[loc_id].Name ) )
						ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s (%s)", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_BAG"], LIGHTYELLOW_FONT_COLOR_CODE, bag_id, bliz_id ) )
						ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s (%s)", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_SLOT"], LIGHTYELLOW_FONT_COLOR_CODE, slot_id, bagtype ) )
						ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s", "sort key", ArkInventory.ItemSortKeyGenerate( i ) ) )
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_CLASS"], LIGHTYELLOW_FONT_COLOR_CODE, class ) )
						
						if i.h then
							
							local class, ilnk, inam, itxt, irar, ilvl, imin, ityp, isub, icount, iloc = ArkInventory.ObjectInfo( i.h )
							
							iloc = getfenv( )[iloc]
							if not iloc then
								iloc = ""
							end

							ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_NAME"], LIGHTYELLOW_FONT_COLOR_CODE, inam ) )
							
							if class == "item" then
							
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s (%s)", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_QUALITY"], LIGHTYELLOW_FONT_COLOR_CODE, irar, getfenv( )["ITEM_QUALITY" .. irar .. "_DESC"] ) )
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_LVL_ITEM"], LIGHTYELLOW_FONT_COLOR_CODE, ilvl ) )
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_LVL_USE"], LIGHTYELLOW_FONT_COLOR_CODE, imin ) )
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_TYPE"], LIGHTYELLOW_FONT_COLOR_CODE, ityp ) )
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_SUBTYPE"], LIGHTYELLOW_FONT_COLOR_CODE, isub ) )
								
								if iloc ~= "" then
									ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_EQUIP"], LIGHTYELLOW_FONT_COLOR_CODE, iloc ) )
								end
								
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_STACK"], LIGHTYELLOW_FONT_COLOR_CODE, icount ) )
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_TEXTURE"], LIGHTYELLOW_FONT_COLOR_CODE, itxt ) )
								
								local ifam = GetItemFamily( i.h )
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_FAMILY"], LIGHTYELLOW_FONT_COLOR_CODE, ifam ) )
							
							elseif class == "spell" then
							
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_TYPE"], LIGHTYELLOW_FONT_COLOR_CODE, string.lower( i.type ) ) )
								
								ArkInventory.Lib.DewDrop:AddLine( )
								
								ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_TEXTURE"], LIGHTYELLOW_FONT_COLOR_CODE, itxt ) )
								
							end
							
						end
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_AI_ID_SHORT"], LIGHTYELLOW_FONT_COLOR_CODE, id ) )
						
						local cid = ArkInventory.ObjectIDCacheCategory( i )
						ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_AI_ID_CACHE"], LIGHTYELLOW_FONT_COLOR_CODE, cid ) )
						
						ArkInventory.Lib.DewDrop:AddLine( "text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_AI_ID_CATEGORY"], LIGHTYELLOW_FONT_COLOR_CODE, cat0.id ) )
						
						if i.h then
							
							if class == "item" then
								local rid = ArkInventory.ObjectIDInternal( i.h )
								rid = strmatch( rid, "^.-%:(.+)" )
								ArkInventory.Lib.DewDrop:AddLine(
									"text", string.format( "%s: %s%s", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_AI_ID_RULE"], LIGHTYELLOW_FONT_COLOR_CODE, rid ),
									"hasArrow", true,
									"hasEditBox", true,
									"editBoxText", rid
								)
							end
							
							ArkInventory.Lib.DewDrop:AddLine( )
							ArkInventory.Lib.DewDrop:AddLine(
								"text", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_PT"],
								"hasArrow", true,
								"tooltipTitle", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_PT"],
								"tooltipText", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_PT_TEXT"],
								"value", "DEBUG_INFO_PT"
							)
							
						end
						
					end
					
					if strsub( value, 1, 16 ) == "CATEGORY_ASSIGN_" then
					
						local k = strmatch( value, "CATEGORY_ASSIGN_(.+)" )
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["CATEGORY_" .. k],
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
					
						for _, v in ArkInventory.spairs( ArkInventory.Global.Category, function(a,b) return ArkInventory.Global.Category[a].fullname < ArkInventory.Global.Category[b].fullname end ) do
					
							local t = v.type_code
							if v.id == cat0 then
								t = "DO_NOT_USE"
							end
							
							--ArkInventory.Output( "id=" .. cat.id .. ", type=[" .. cat.type .. "], value=[" .. value .. "]" )
							if k == t then
								--~~~~
								ArkInventory.Lib.DewDrop:AddLine(
									"text", v.name,
									"tooltipTitle", "Assign to Category",
									"tooltipText", "assign this item to " .. v.fullname, --~~~~ check this
									"closeWhenClicked", true,
									"func", function( )
										ArkInventory.ItemCategorySet( i, v.id )
										ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
									end
								)
							
							end
							
						end
						
						if k == "CUSTOM" then
						
							ArkInventory.Lib.DewDrop:AddLine( )
							
							ArkInventory.Lib.DewDrop:AddLine(
								"text", ArkInventory.Localise["MOD_MENU_ITEM_CUSTOM_NEW"],
								"hasArrow", true,
								"hasEditBox", true,
								"editBoxText", "",
								"closeWhenClicked", true,
								"editBoxFunc", function( v )
									ArkInventory.CategoryCustomAdd( v )
								end
							)
							
						end
						
					end
					
				end
				
				
				if level == 3 and value then
					
					if value == "DEBUG_INFO_PT" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_PT_TITLE"] .. ":",
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						--local pt_set = ArkInventory.Lib.PeriodicTable:ItemSearch( i.h )
						local pt_set = ArkInventory.PTItemSearch( i.h )
						
						if not pt_set then
						
							ArkInventory.Lib.DewDrop:AddLine( "text", LIGHTYELLOW_FONT_COLOR_CODE .. ArkInventory.Localise["MOD_MENU_ITEM_DEBUG_PT_NONE"] )
						
						else
						
							for k, v in pairs( pt_set ) do
								ArkInventory.Lib.DewDrop:AddLine(
									"text", v,
									"hasArrow", true,
									"hasEditBox", true,
									"editBoxText", v
								)
							end
							
						end
						
					end
				
				end
				
			end
			
		)
		
	end
	
end


function ArkInventory.MenuBagOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
		
	else

		local loc_id = frame.ARK_Data.loc_id
		local bag_id = frame.ARK_Data.bag_id
		local cp = ArkInventory.LocationPlayerInfoGet( loc_id )
		local player_id = cp.info.player_id
		
		local bag = cp.location[loc_id].bag[bag_id]
		
		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "BOTTOMLEFT"
		else
			p = "TOPLEFT"
			rp = "BOTTOMRIGHT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
			
				if level == 1 then

					ArkInventory.Lib.DewDrop:AddLine(
						--"text", string.upper( string.format( ArkInventory.Localise["MOD_MENU_BAG_TITLE"], bag_id ) ),
						"text", ArkInventory.Localise["MOD_MENU_BAG_TITLE"],
						"isTitle", true,
						"icon", bag.texture,
						"textHeight", 12
					)
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_BAG_SHOW"],
						"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAG_SHOW"],
						"tooltipText", ArkInventory.Localise["MOD_MENU_BAG_SHOW_TEXT"],
						"checked", ArkInventory.db.factionrealm.player.data[player_id].display[loc_id].bag[bag_id],
						"func", function( )
							ArkInventory.db.factionrealm.player.data[player_id].display[loc_id].bag[bag_id] = not ArkInventory.db.factionrealm.player.data[player_id].display[loc_id].bag[bag_id]
							ArkInventory.Lib.DewDrop:Close( )
							ArkInventory.Frame_Main_Generate( loc_id, ArkInventory.Const.Window.Draw.Recalculate )
						end
					)

					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_BAG_ISOLATE"],
						"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAG_ISOLATE"],
						"tooltipText", ArkInventory.Localise["MOD_MENU_BAG_ISOLATE_TEXT"],
						"func", function( )
							for x in pairs( ArkInventory.Global.Location[loc_id].Bags ) do
								if x == bag_id then
									ArkInventory.db.factionrealm.player.data[player_id].display[loc_id].bag[x] = true
								else
									ArkInventory.db.factionrealm.player.data[player_id].display[loc_id].bag[x] = false
								end
							end
							ArkInventory.Lib.DewDrop:Close( )
							ArkInventory.Frame_Main_Generate( loc_id, ArkInventory.Const.Window.Draw.Recalculate )
						end
					)

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_BAG_SHOWALL"],
						"tooltipTitle", ArkInventory.Localise["MOD_MENU_BAG_SHOWALL"],
						"tooltipText", ArkInventory.Localise["MOD_MENU_BAG_SHOWALL_TEXT"],
						"func", function( )
							for x in pairs( ArkInventory.Global.Location[loc_id].Bags ) do
								ArkInventory.db.factionrealm.player.data[player_id].display[loc_id].bag[x] = true
							end
							ArkInventory.Lib.DewDrop:Close( )
							ArkInventory.Frame_Main_Generate( loc_id, ArkInventory.Const.Window.Draw.Recalculate )
						end
					)
					
				end

			end
		)
		
	end
	
end

function ArkInventory.MenuVaultTabOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
		
	else

		local loc_id = frame.ARK_Data.loc_id
		local bag_id = frame.ARK_Data.bag_id
		local cp = ArkInventory.LocationPlayerInfoGet( loc_id )
		local bag = cp.location[loc_id].bag[bag_id]
		local button = getfenv( )[ArkInventory.Const.Frame.Main.Name .. loc_id .. ArkInventory.Const.Frame.Changer.Name .. "WindowBag" .. bag_id]

		
		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "TOPLEFT"
		else
			p = "TOPLEFT"
			rp = "TOPRIGHT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
			
				if level == 1 then
				
					ArkInventory.Lib.DewDrop:AddLine(
						"text", string.format( "%s: %s", GUILD_BANK, string.format( GUILDBANK_TAB_NUMBER, bag_id ) ),
						"isTitle", true,
						"icon", bag.texture,
						"textHeight", 12
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", bag.name,
						"isTitle", true,
						"textHeight", 12
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", "request tab data",
						"closeWhenClicked", true,
						"func", function( )
							ArkInventory.ScanVault( )
						end
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", "mode: " .. GUILD_BANK,
						"closeWhenClicked", true,
						"disabled", GuildBankFrame.mode == "bank",
						"func", function( )
							GuildBankFrame.mode = "bank"
							ArkInventory.Frame_Changer_Vault_Tab_OnClick( button, "LeftButton" )
							ArkInventory.Frame_Main_Generate( loc_id, ArkInventory.Const.Window.Draw.Recalculate )
						end
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", "mode: " .. GUILD_BANK_LOG,
						"closeWhenClicked", true,
						"disabled", GuildBankFrame.mode == "log",
						"func", function( )
							GuildBankFrame.mode = "log"
							ArkInventory.Frame_Changer_Vault_Tab_OnClick( button, "LeftButton" )
							ArkInventory.Frame_Main_Generate( loc_id, ArkInventory.Const.Window.Draw.Recalculate )
						end
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", "mode: " .. GUILD_BANK_MONEY_LOG,
						"closeWhenClicked", true,
						"disabled", GuildBankFrame.mode == "moneylog",
						"func", function( )
							GuildBankFrame.mode = "moneylog"
							ArkInventory.Frame_Changer_Vault_Tab_OnClick( button, "LeftButton" )
							ArkInventory.Frame_Main_Generate( loc_id, ArkInventory.Const.Window.Draw.Recalculate )
						end
					)
					
					if IsGuildLeader( ) then
					
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", "change name or icon",
							"closeWhenClicked", true,
							"func", function( )
								SetCurrentGuildBankTab( bag_id )
								GuildBankPopupFrame:Show( )
								GuildBankPopupFrame_Update( bag_id )
							end
						)
						
					end
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
			
			end
			
		)
		
	end
	
end


function ArkInventory.MenuSearchOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "TOPLEFT"
		else
			p = "TOPLEFT"
			rp = "TOPRIGHT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
			
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_MAIN_SEARCH"],
						"isTitle", true,
						"textHeight", 14
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_SCALE"],
						"tooltipTitle", ArkInventory.Localise["GENERIC_SCALE"],
						"hasArrow", true,
						"hasSlider", true,
						"sliderIsPercent", true,
						"sliderValue", ArkInventory.db.profile.option.ui.search.scale,
						"sliderMin", 0.4,
						"sliderMax", 2,
						"sliderStep", 0.05,
						"sliderFunc", function( v )
							if v ~= ArkInventory.db.profile.option.ui.search.scale then
								ArkInventory.db.profile.option.ui.search.scale = v
								ArkInventory.Frame_Search_Paint( )
							end
						end
					)

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_FONT"],
						"hasArrow", true,
						"value", "SEARCH_FONT_FACE"
					)

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_BORDER"],
						"hasArrow", true,
						"value", "SEARCH_BORDER"
					)
					
				end


				if level == 2 and value then
				
					if value == "SEARCH_FONT_FACE" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_FONT"],
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						for _, fontName in pairs( ArkInventory.Lib.SharedMedia:List( "font" ) ) do
							ArkInventory.Lib.DewDrop:AddLine(
								"text", fontName,
								"tooltipTitle", ArkInventory.Localise["GENERIC_FONT"],
								--"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_FONT_TEXT"], fontName ),
								"checked", ArkInventory.db.profile.option.ui.search.font.name == fontName,
								"func", function( )
									ArkInventory.db.profile.option.ui.search.font.name = fontName
									ArkInventory.Frame_Search_Paint( )
								end
							)
						end
						
					end


					if value == "SEARCH_BORDER" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_BORDER"],
							"isTitle", true,
							"textHeight", 12
						)
						ArkInventory.Lib.DewDrop:AddLine( )

						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_SHOW"],
							"tooltipTitle", ArkInventory.Localise["GENERIC_SHOW"],
							--"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER_SHOW_TEXT"],
							"checked", ArkInventory.db.profile.option.ui.search.border.show,
							"func", function( )
								ArkInventory.db.profile.option.ui.search.border.show = not ArkInventory.db.profile.option.ui.search.border.show
								ArkInventory.Frame_Search_Paint( )
							end
						)

						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_COLOUR"],
							"tooltipTitle", ArkInventory.Localise["GENERIC_COLOUR"],
							--"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER_COLOR_TEXT"],
							"disabled", not ArkInventory.db.profile.option.ui.search.border.show,
							"hasColorSwatch", true,
							"r", ArkInventory.db.profile.option.ui.search.border.colour.r,
							"g", ArkInventory.db.profile.option.ui.search.border.colour.g,
							"b", ArkInventory.db.profile.option.ui.search.border.colour.b,
							"colorFunc", function( r, g, b )
								ArkInventory.db.profile.option.ui.search.border.colour.r = r
								ArkInventory.db.profile.option.ui.search.border.colour.g = g
								ArkInventory.db.profile.option.ui.search.border.colour.b = b
								ArkInventory.Frame_Search_Paint( )
							end
						)

						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_TEXTURE"],
							"tooltipTitle", ArkInventory.Localise["GENERIC_TEXTURE"],
							--"tooltipText", ArkInventory.Localise["GENERIC_BORDER_TEXTURE_TEXT"],
							"hasArrow", true,
							"value", "SEARCH_BORDER_TEXTURE"
						)
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_SCALE"],
							"tooltipTitle", ArkInventory.Localise["GENERIC_SCALE"],
							--"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_BORDER_SCALE_TEXT"],
							"hasArrow", true,
							"hasSlider", true,
							"sliderIsPercent", true,
							"sliderValue", ArkInventory.db.profile.option.ui.search.border.scale or 1,
							"sliderMin", 0.25,
							"sliderMax", 4,
							"sliderStep", 0.05,
							"sliderFunc", function( v )
								if ArkInventory.db.profile.option.ui.search.border.scale ~= v then
									ArkInventory.db.profile.option.ui.search.border.scale = v
									ArkInventory.Frame_Search_Paint( )
								end
							end
						)


					end

					
				end



				if level == 3 and value then
				
					if value == "SEARCH_BORDER_TEXTURE" then

						local t = ""
						local default = "Tooltip"
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_TEXTURE"],
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )

						t = ArkInventory.db.profile.option.ui.search.file or ArkInventory.Const.Texture.Border[default].file
						ArkInventory.Lib.DewDrop:AddLine(
							"text", string.format( "%s: %s", ArkInventory.Localise["GENERIC_FILE"], t ),
							"tooltipTitle", ArkInventory.Localise["GENERIC_FILE"],
							"tooltipText", ArkInventory.Localise["GENERIC_BORDER_TEXTURE_FILE_TEXT"],
							"hasArrow", true,
							"hasEditBox", true,
							"editBoxText", t,
							"editBoxFunc", function( v )
								ArkInventory.db.profile.option.ui.search.border.file = v
								ArkInventory.db.profile.option.ui.search.border.scale = 1
								ArkInventory.Frame_Search_Paint( )
							end
						)
						
						t = ArkInventory.db.profile.option.ui.search.border.size or ArkInventory.Const.Texture.Border[default].size
						ArkInventory.Lib.DewDrop:AddLine(
							"text", string.format( "%s: %s", ArkInventory.Localise["GENERIC_HEIGHT"], t ),
							"tooltipTitle", ArkInventory.Localise["GENERIC_HEIGHT"],
							"tooltipText", ArkInventory.Localise["GENERIC_BORDER_TEXTURE_HEIGHT_TEXT"],
							"hasArrow", true,
							"hasEditBox", true,
							"editBoxText", t,
							"editBoxValidateFunc", function( v )
								return not string.find( v, "%D" )
							end,
							"editBoxFunc", function( v )
								ArkInventory.db.profile.option.ui.search.border.size = tonumber( v )
								ArkInventory.Frame_Search_Paint( )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						ArkInventory.Lib.DewDrop:AddLine(
							"text", "Default Styles",
							"isTitle", true
						)
						
						local styles = { }
						for style in pairs( ArkInventory.Const.Texture.Border ) do
							table.insert( styles, style )
						end
						table.sort( styles )
						
						for _, style in pairs( styles ) do
						
							ArkInventory.Lib.DewDrop:AddLine(
								"text", style,
								"func", function( )
									ArkInventory.db.profile.option.ui.search.border.file = ArkInventory.Const.Texture.Border[style].file
									ArkInventory.db.profile.option.ui.search.border.size = ArkInventory.Const.Texture.Border[style].size
									ArkInventory.db.profile.option.ui.search.border.scale = 1
									ArkInventory.Frame_Search_Paint( )
								end
							)
							
						end
						
					end

				end

				
				
				if level == 1 then
					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
				end

			end
			
		)

	end
	
	
	
end


function ArkInventory.MenuRulesOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "TOPLEFT"
		else
			p = "TOPLEFT"
			rp = "TOPRIGHT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
			
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_MAIN_RULES"],
						"isTitle", true,
						"textHeight", 14
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_SCALE"],
						"tooltipTitle", ArkInventory.Localise["GENERIC_SCALE"],
						"hasArrow", true,
						"hasSlider", true,
						"sliderIsPercent", true,
						"sliderValue", ArkInventory.db.profile.option.ui.rules.scale,
						"sliderMin", 0.4,
						"sliderMax", 2,
						"sliderStep", 0.05,
						"sliderFunc", function( v )
							if v ~= ArkInventory.db.profile.option.ui.rules.scale then
								ArkInventory.db.profile.option.ui.rules.scale = v
								ArkInventory.Frame_Rules_Paint( )
							end
						end
					)

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_FONT"],
						"hasArrow", true,
						"value", "RULES_FONT_FACE"
					)

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_BORDER"],
						"hasArrow", true,
						"value", "RULES_BORDER"
					)

					ArkInventory.Lib.DewDrop:AddLine( )

					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_RULE_SHOWDISABLED"],
						"tooltipTitle", ArkInventory.Localise["MOD_MENU_RULE_SHOWDISABLED"],
						"tooltipText", ArkInventory.Localise["MOD_MENU_RULE_SHOWDISABLED_TEXT"],
						"checked", ArkInventory.db.global.option.showdisabled,
						"closeWhenClicked", true,
						"func", function( )
							ArkInventory.db.global.option.showdisabled = not ArkInventory.db.global.option.showdisabled,
							ArkInventory.Frame_Rules_Table_Reset( "ARKINV_RulesFrameView" )
							ArkInventory.Frame_Rules_Table_Refresh( "ARKINV_RulesFrame" )
						end
					)
					
				end


				if level == 2 and value then
				
					if value == "RULES_FONT_FACE" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_FONT"],
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						for _, fontName in pairs( ArkInventory.Lib.SharedMedia:List( "font" ) ) do
							ArkInventory.Lib.DewDrop:AddLine(
								"text", fontName,
								"tooltipTitle", ArkInventory.Localise["GENERIC_FONT"],
								--"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_FONT_TEXT"], fontName ),
								"checked", ArkInventory.db.profile.option.ui.rules.font.name == fontName,
								"func", function( )
									ArkInventory.db.profile.option.ui.rules.font.name = fontName
									ArkInventory.Frame_Rules_Paint( )
								end
							)
						end
						
					end


					if value == "RULES_BORDER" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_BORDER"],
							"isTitle", true,
							"textHeight", 12
						)
						ArkInventory.Lib.DewDrop:AddLine( )

						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_SHOW"],
							"tooltipTitle", ArkInventory.Localise["GENERIC_SHOW"],
							--"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER_SHOW_TEXT"],
							"checked", ArkInventory.db.profile.option.ui.rules.border.show,
							"func", function( )
								ArkInventory.db.profile.option.ui.rules.border.show = not ArkInventory.db.profile.option.ui.rules.border.show
								ArkInventory.Frame_Rules_Paint( )
							end
						)

						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_COLOUR"],
							"tooltipTitle", ArkInventory.Localise["GENERIC_COLOUR"],
							--"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER_COLOR_TEXT"],
							"disabled", not ArkInventory.db.profile.option.ui.rules.border.show,
							"hasColorSwatch", true,
							"r", ArkInventory.db.profile.option.ui.rules.border.colour.r,
							"g", ArkInventory.db.profile.option.ui.rules.border.colour.g,
							"b", ArkInventory.db.profile.option.ui.rules.border.colour.b,
							"colorFunc", function( r, g, b )
								ArkInventory.db.profile.option.ui.rules.border.colour.r = r
								ArkInventory.db.profile.option.ui.rules.border.colour.g = g
								ArkInventory.db.profile.option.ui.rules.border.colour.b = b
								ArkInventory.Frame_Rules_Paint( )
							end
						)

						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_TEXTURE"],
							"tooltipTitle", ArkInventory.Localise["GENERIC_TEXTURE"],
							--"tooltipText", ArkInventory.Localise["GENERIC_BORDER_TEXTURE_TEXT"],
							"hasArrow", true,
							"value", "RULES_BORDER_TEXTURE"
						)
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_SCALE"],
							"tooltipTitle", ArkInventory.Localise["GENERIC_SCALE"],
							--"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_BORDER_SCALE_TEXT"],
							"hasArrow", true,
							"hasSlider", true,
							"sliderIsPercent", true,
							"sliderValue", ArkInventory.db.profile.option.ui.rules.border.scale or 1,
							"sliderMin", 0.25,
							"sliderMax", 4,
							"sliderStep", 0.05,
							"sliderFunc", function( v )
								if ArkInventory.db.profile.option.ui.rules.border.scale ~= v then
									ArkInventory.db.profile.option.ui.rules.border.scale = v
									ArkInventory.Frame_Rules_Paint( )
								end
							end
						)


					end

					
				end



				if level == 3 and value then
				
					if value == "RULES_BORDER_TEXTURE" then

						local t = ""
						local default = "Tooltip"
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_TEXTURE"],
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )

						t = ArkInventory.db.profile.option.ui.rules.file or ArkInventory.Const.Texture.Border[default].file
						ArkInventory.Lib.DewDrop:AddLine(
							"text", string.format( "%s: %s", ArkInventory.Localise["GENERIC_FILE"], t ),
							"tooltipTitle", ArkInventory.Localise["GENERIC_FILE"],
							"tooltipText", ArkInventory.Localise["GENERIC_BORDER_TEXTURE_FILE_TEXT"],
							"hasArrow", true,
							"hasEditBox", true,
							"editBoxText", t,
							"editBoxFunc", function( v )
								ArkInventory.db.profile.option.ui.rules.border.file = v
								ArkInventory.db.profile.option.ui.rules.border.scale = 1
								ArkInventory.Frame_Rules_Paint( )
							end
						)
						
						t = ArkInventory.db.profile.option.ui.rules.border.size or ArkInventory.Const.Texture.Border[default].size
						ArkInventory.Lib.DewDrop:AddLine(
							"text", string.format( "%s: %s", ArkInventory.Localise["GENERIC_HEIGHT"], t ),
							"tooltipTitle", ArkInventory.Localise["GENERIC_HEIGHT"],
							"tooltipText", ArkInventory.Localise["GENERIC_BORDER_TEXTURE_HEIGHT_TEXT"],
							"hasArrow", true,
							"hasEditBox", true,
							"editBoxText", t,
							"editBoxValidateFunc", function( v )
								return not string.find( v, "%D" )
							end,
							"editBoxFunc", function( v )
								ArkInventory.db.profile.option.ui.rules.border.size = tonumber( v )
								ArkInventory.Frame_Rules_Paint( )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						ArkInventory.Lib.DewDrop:AddLine(
							"text", "Default Styles",
							"isTitle", true
						)
						
						local styles = { }
						for style in pairs( ArkInventory.Const.Texture.Border ) do
							table.insert( styles, style )
						end
						table.sort( styles )
						
						for _, style in pairs( styles ) do
						
							ArkInventory.Lib.DewDrop:AddLine(
								"text", style,
								"func", function( )
									ArkInventory.db.profile.option.ui.rules.border.file = ArkInventory.Const.Texture.Border[style].file
									ArkInventory.db.profile.option.ui.rules.border.size = ArkInventory.Const.Texture.Border[style].size
									ArkInventory.db.profile.option.ui.rules.border.scale = 1
									ArkInventory.Frame_Rules_Paint( )
								end
							)
							
						end
						
					end

				end

				
				if level == 1 then
					ArkInventory.Lib.DewDrop:AddLine( )
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
				end

			end
			
		)

	end
	
	
	
end



function ArkInventory.MenuSwitchLocation( frame, level, value, offset )

	ArkInventory.Lib.DewDrop:AddLine(
		"text", ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_LOCATION"],
		"isTitle", true,
		"textHeight", 12
	)
	
	if level == offset + 1 then
	
		for set_id, loc in ipairs( ArkInventory.Global.Location ) do
			if ArkInventory.Global.Location[set_id].canView then
				ArkInventory.Lib.DewDrop:AddLine(
					"text", loc.Name,
					"tooltipTitle", loc.Name,
					"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_LOCATION_TEXT"], loc.Name ),
					"icon", loc.Texture,
					"closeWhenClicked", true,
					"func", function( )
						ArkInventory.Frame_Main_Toggle( set_id )
					end
				)
			end
		end
		
	end
	
end

function ArkInventory.MenuSwitchLocationOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "BOTTOMLEFT"
		else
			p = "TOPLEFT"
			rp = "BOTTOMRIGHT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
			
				ArkInventory.MenuSwitchLocation( frame, level, value, 0 )
				
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
			end
		)

	end
	
end

function ArkInventory.MenuSwitchCharacter( frame, level, value, offset )
	
	local loc_id = frame.ARK_Data.loc_id
	local cp = ArkInventory.LocationPlayerInfoGet( loc_id )
	
	if level == offset + 1 then
	
		ArkInventory.Lib.DewDrop:AddLine(
			"text", ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_CHARACTER"],
			"isTitle", true,
			"textHeight", 12
		)
		
		local fc = 0
		
			local count = 0
		
			for n, tp in ArkInventory.spairs( ArkInventory.db.factionrealm.player.data, function(a,b) return a < b end ) do
			
				local show = false
				
				if loc_id ~= ArkInventory.Const.Location.Vault then
					
					if string.sub( n, 1, 1 ) ~= ArkInventory.Const.GuildTag then
						show = true
					end
					
				else
				
					tp = ArkInventory.PlayerInfoGet( tp.info.guild_id )
					
					if string.sub( n, 1, 1 ) == ArkInventory.Const.GuildTag then
						show = true
					end
				
				end
				
				if not tp or tp.location[loc_id].slot_count == 0 then
					show = false
				end
				
				if show then
					
					count = count + 1
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.DisplayName4( tp.info ),
						--"tooltipTitle", "",
						--"tooltipText", "",
						"hasArrow", true,
						"isRadio", true,
						"checked", cp.info.player_id == tp.info.player_id,
						--"notClickable", cp.info.player_id == tp.info.player_id,
						"closeWhenClicked", true,
						"func", function( )
							ArkInventory.Frame_Main_Show( loc_id, tp.info.player_id )
						end,
						"value", ( "SWITCH_CHARACTER_SWITCH_" .. tp.info.player_id )
					)
					
				end
				
			end
			
			
			if count == 0 then
			
				ArkInventory.Lib.DewDrop:AddLine(
					"text", "no data availale",
					"disabled", true
				)
				
			end
			
		end
	
	if level == offset + 2 and value then

		local player_id = strmatch( value, "^SWITCH_CHARACTER_SWITCH_(.+)" )
		local tp = ArkInventory.PlayerInfoGet( player_id )
		
		ArkInventory.Lib.DewDrop:AddLine(
			"text", ArkInventory.DisplayName4( tp.info ),
			"isTitle", true,
			"textHeight", 12
		)
		
		ArkInventory.Lib.DewDrop:AddLine( )
		
		ArkInventory.Lib.DewDrop:AddLine(
			"text", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_CHARACTER_ERASE"], ArkInventory.Global.Location[loc_id].Name ),
			"tooltipTitle", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_CHARACTER_ERASE"], ArkInventory.Global.Location[loc_id].Name ),
			"tooltipText", RED_FONT_COLOR_CODE .. string.format( ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_CHARACTER_ERASE_TEXT"], ArkInventory.Global.Location[loc_id].Name, ArkInventory.DisplayName1( tp.info ) ),
			"closeWhenClicked", true,
			"func", function( )
				ArkInventory.Frame_Main_Hide( loc_id )
				ArkInventory.EraseSavedData( tp.info.player_id, loc_id )
			end
		)
		
		ArkInventory.Lib.DewDrop:AddLine(
			"text", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_CHARACTER_ERASE"], ArkInventory.Localise["LOCATION_ALL"] ),
			"tooltipTitle", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_CHARACTER_ERASE"], ArkInventory.Localise["LOCATION_ALL"] ),
			"tooltipText", RED_FONT_COLOR_CODE .. string.format( ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_CHARACTER_ERASE_TEXT"], ArkInventory.Localise["LOCATION_ALL"], ArkInventory.DisplayName1( tp.info ) ),
			"closeWhenClicked", true,
			"func", function( )
				ArkInventory.Frame_Main_Hide( )
				ArkInventory.EraseSavedData( tp.info.player_id )
			end
		)
	
	end

end

function ArkInventory.MenuSwitchCharacterOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetLeft( ) + ( this:GetRight( ) - this:GetLeft( ) ) / 2
		if ( x >= ( GetScreenWidth( ) / 2 ) ) then
			p = "TOPRIGHT"
			rp = "BOTTOMLEFT"
		else
			p = "TOPLEFT"
			rp = "BOTTOMRIGHT"
		end
		
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
				
				ArkInventory.MenuSwitchCharacter( frame, level, value, 0 )
				
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
			end
			
		)
	
	end
	
end

function ArkInventory.MenuLDBBagsOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetBottom( ) + ( this:GetTop( ) - this:GetBottom( ) ) / 2
		if ( x >= ( GetScreenHeight( ) / 2 ) ) then
			p = "TOPLEFT"
			rp = "BOTTOMLEFT"
		else
			p = "BOTTOMLEFT"
			rp = "TOPLEFT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
				
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Const.Program.Name,
						"isTitle", true,
						"textHeight", 14
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Global.Version,
						"notClickable", true
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["OPT_CONFIG"],
						"closeWhenClicked", true,
						"func", function( )
							ArkInventory.Lib.Dialog:Open( ArkInventory.Const.Frame.Options.Internal )
						end
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_MAIN_ACTIONS"],
						"hasArrow", true,
						"value", "ACTIONS"
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_MAIN_SWITCH_LOCATION"],
						"hasArrow", true,
						"value", "LOCATION"
					)
						
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_FONT"],
						"hasArrow", true,
						"value", "FONT"
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["LDB"],
						"hasArrow", true,
						"value", "LDB"
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
				
				if level == 2 and value then
				
					if value == "LOCATION" then
						ArkInventory.MenuSwitchLocation( frame, level, value, 1 )
					end
				
					if value == "FONT" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["GENERIC_FONT"],
							"isTitle", true,
							"textHeight", 12
						)
						
						for _, fontName in pairs( ArkInventory.Lib.SharedMedia:List( "font" ) ) do
							ArkInventory.Lib.DewDrop:AddLine(
								"text", fontName,
								"tooltipTitle", ArkInventory.Localise["GENERIC_FONT"],
								"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_FONT_TEXT"], fontName ),
								"checked", fontName == ArkInventory.db.profile.option.font.name,
								"func", function( )
									ArkInventory.SetFontAll( fontName )
								end
							)
						end
						
					end
					
					if value == "ACTIONS" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_MAIN_ACTIONS"],
							"isTitle", true,
							"textHeight", 12
						)
						
						for k, v in pairs( ArkInventory.Const.Actions ) do
							if v.LDB then
								ArkInventory.Lib.DewDrop:AddLine(
									"text", v.Name,
									"closeWhenClicked", true,
									"icon", v.Texture,
									"func", v.Scripts.OnClick
								)
							end
						end
						
					end
					
					if value == "LDB" then
					
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["LDB"],
							"isTitle", true,
							"textHeight", 12
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_LDB_BAGS_COLOUR_USE"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_LDB_BAGS_COLOUR_USE"],
							--"tooltipText", "tooltipText",
							"checked", ArkInventory.db.char.option.ldb.bags.colour,
							"func", function( )
								ArkInventory.db.char.option.ldb.bags.colour = not ArkInventory.db.char.option.ldb.bags.colour
								ArkInventory.LDB.Bags:Update( )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_LDB_BAGS_STYLE"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_LDB_BAGS_STYLE"],
							--"tooltipText", "tooltipText",
							"checked", ArkInventory.db.char.option.ldb.bags.full,
							"func", function( )
								ArkInventory.db.char.option.ldb.bags.full = not ArkInventory.db.char.option.ldb.bags.full
								ArkInventory.LDB.Bags:Update( )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", ArkInventory.Localise["MOD_MENU_LDB_BAGS_INCLUDE_TYPE"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_LDB_BAGS_INCLUDE_TYPE"],
							--"tooltipText", "tooltipText",
							"checked", ArkInventory.db.char.option.ldb.bags.includetype,
							"func", function( )
								ArkInventory.db.char.option.ldb.bags.includetype = not ArkInventory.db.char.option.ldb.bags.includetype
								ArkInventory.LDB.Bags:Update( )
							end
						)
						
					end
					
					
				end
				
			end
			
		)
	
	end
	
end

function ArkInventory.MenuLDBAmmoOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetBottom( ) + ( this:GetTop( ) - this:GetBottom( ) ) / 2
		if ( x >= ( GetScreenHeight( ) / 2 ) ) then
			p = "TOPLEFT"
			rp = "BOTTOMLEFT"
		else
			p = "BOTTOMLEFT"
			rp = "TOPLEFT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
				
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.LDB.Ammo.name,
						"isTitle", true,
						"textHeight", 14
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Global.Version,
						"notClickable", true
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["MOD_MENU_LDB_AMMO_INCLUDE_DURABILITY"],
						"tooltipTitle", ArkInventory.Localise["MOD_MENU_LDB_AMMO_INCLUDE_DURABILITY"],
						--"tooltipText", "tooltipText",
						"checked", ArkInventory.db.char.option.ldb.ammo.durability,
						"func", function( )
							ArkInventory.db.char.option.ldb.ammo.durability = not ArkInventory.db.char.option.ldb.ammo.durability
							ArkInventory.LDB.Ammo:Update( )
						end
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
			end
			
		)
	
	end
	
end

function ArkInventory.MenuLDBCurrencyOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetBottom( ) + ( this:GetTop( ) - this:GetBottom( ) ) / 2
		if ( x >= ( GetScreenHeight( ) / 2 ) ) then
			p = "TOPLEFT"
			rp = "BOTTOMLEFT"
		else
			p = "BOTTOMLEFT"
			rp = "TOPLEFT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
				
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.LDB.Currency.name,
						"isTitle", true,
						"textHeight", 14
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Global.Version,
						"notClickable", true
					)
					
					-- expand all token headers
					local numTokenTypes = GetCurrencyListSize( )
	
					if numTokenTypes > 0 then
						
						for j = numTokenTypes, 1, -1 do
							local _, isHeader, isExpanded = GetCurrencyListInfo( j )
							if isHeader and not isExpanded then
								ExpandCurrencyList( j, 1 )
							end
						end
						
						local numTokenTypes = GetCurrencyListSize( )
						
						for j = 1, numTokenTypes do
						
							local name, isHeader, _, _, _, count, _, _, item = GetCurrencyListInfo( j )
							
							if isHeader then
								
								ArkInventory.Lib.DewDrop:AddLine( )
								ArkInventory.Lib.DewDrop:AddLine(
									"text", name,
									"isTitle", true
								)
								
							else
								
								local checked = ArkInventory.db.char.option.ldb.currency.track == name
								local tooltipText = ArkInventory.Localise["MOD_MENU_LDB_CURRENCY_SELECT"]
								
								if checked then
									name = "|cff00ff00" .. name
									tooltipText = ArkInventory.Localise["MOD_MENU_LDB_CURRENCY_DESELECT"]
								end
								
								ArkInventory.Lib.DewDrop:AddLine(
									"icon", GetItemIcon( item ),
									"text", name,
									"tooltipTitle", name,
									"tooltipText", tooltipText,
									"checked", checked,
									"func", function( )
										if checked then
											ArkInventory.db.char.option.ldb.currency.track = nil
										else
											ArkInventory.db.char.option.ldb.currency.track = name
										end
										ArkInventory.LDB.Currency:Update( )
									end
								)
								
							end
							
						end
						
					end
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
			end
			
		)
	
	end
	
end


function ArkInventory.MenuLDBPetsOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetBottom( ) + ( this:GetTop( ) - this:GetBottom( ) ) / 2
		if ( x >= ( GetScreenHeight( ) / 2 ) ) then
			p = "TOPLEFT"
			rp = "BOTTOMLEFT"
		else
			p = "BOTTOMLEFT"
			rp = "TOPLEFT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
				
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.LDB.Pets.name,
						"isTitle", true,
						"textHeight", 14
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Global.Version,
						"notClickable", true
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					local type_id = "CRITTER"
					local n = GetNumCompanions( type_id )
					
					for k = 1, n do
						
						local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
						
						local checked = ArkInventory.db.char.option.ldb.pets.track == creatureSpellID
						local tooltipText = ArkInventory.Localise["MOD_MENU_LDB_PETS_SELECT"]
						
						if checked then
							creatureName = "|cff00ff00" .. creatureName
							tooltipText = ArkInventory.Localise["MOD_MENU_LDB_PETS_DESELECT"]
						end
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", creatureName,
							"tooltipTitle", creatureName,
							"tooltipText", tooltipText,
							"checked", checked,
							"func", function( )
								if checked then
									ArkInventory.db.char.option.ldb.pets.track = nil
								else
									ArkInventory.db.char.option.ldb.pets.track = creatureSpellID
								end
								ArkInventory.LDB.Pets:Update( )
							end
						)
						
					end
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					if n > 1 then
						
						local creatureSpellID = 0
						local creatureName = ArkInventory.Localise["GENERIC_RANDOM"]
						local checked = ArkInventory.db.char.option.ldb.pets.track == creatureSpellID
						local tooltipText = ArkInventory.Localise["MOD_MENU_LDB_PETS_SELECT"]
						
						if checked then
							creatureName = "|cff00ff00" .. creatureName
							tooltipText = ArkInventory.Localise["MOD_MENU_LDB_PETS_DESELECT"]
						end
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", creatureName,
							"tooltipTitle", creatureName,
							"tooltipText", tooltipText,
							"checked", checked,
							"func", function( )
								if checked then
									ArkInventory.db.char.option.ldb.pets.track = nil
								else
									ArkInventory.db.char.option.ldb.pets.track = creatureSpellID
								end
								ArkInventory.LDB.Pets:Update( )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
					end
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
			end
			
		)
	
	end
	
end


function ArkInventory.MenuLDBMountsOpen( frame )

	if not frame then
		frame = this
	end

	if ArkInventory.Lib.DewDrop:IsOpen( this ) then
	
		ArkInventory.Lib.DewDrop:Close( )
	
	else

		local x, p, rp
		x = this:GetBottom( ) + ( this:GetTop( ) - this:GetBottom( ) ) / 2
		if ( x >= ( GetScreenHeight( ) / 2 ) ) then
			p = "TOPLEFT"
			rp = "BOTTOMLEFT"
		else
			p = "BOTTOMLEFT"
			rp = "TOPLEFT"
		end
	
		ArkInventory.Lib.DewDrop:Open( this,
			"point", p,
			"relativePoint", rp,
			"children", function( level, value )
				
				if level == 1 then
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.LDB.Mounts.name,
						"isTitle", true,
						"textHeight", 14
					)
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Global.Version,
						"notClickable", true
					)
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					local type_id = "MOUNT"
					local n = GetNumCompanions( type_id )
					
					for k = 1, n do
						
						local creatureID, creatureName, creatureSpellID, texture, active = GetCompanionInfo( type_id, k )
						
						local checked = ArkInventory.db.char.option.ldb.mounts.track == creatureSpellID
						local tooltipText = ArkInventory.Localise["MOD_MENU_LDB_MOUNTS_SELECT"]
						
						if checked then
							creatureName = "|cff00ff00" .. creatureName
							tooltipText = ArkInventory.Localise["MOD_MENU_LDB_MOUNTS_DESELECT"]
						end
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", creatureName,
							"tooltipTitle", creatureName,
							"tooltipText", tooltipText,
							"checked", checked,
							"func", function( )
								if checked then
									ArkInventory.db.char.option.ldb.mounts.track = nil
								else
									ArkInventory.db.char.option.ldb.mounts.track = creatureSpellID
								end
								ArkInventory.LDB.Mounts:Update( )
							end
						)
						
					end
					
					ArkInventory.Lib.DewDrop:AddLine( )
					
					if n > 1 then
						
						local creatureSpellID = 0
						local creatureName = ArkInventory.Localise["GENERIC_RANDOM"]
						local checked = ArkInventory.db.char.option.ldb.mounts.track == creatureSpellID
						local tooltipText = ArkInventory.Localise["MOD_MENU_LDB_MOUNTS_SELECT"]
						
						if checked then
							creatureName = "|cff00ff00" .. creatureName
							tooltipText = ArkInventory.Localise["MOD_MENU_LDB_MOUNTS_DESELECT"]
						end
						
						ArkInventory.Lib.DewDrop:AddLine(
							"text", creatureName,
							"tooltipTitle", creatureName,
							"tooltipText", tooltipText,
							"checked", checked,
							"func", function( )
								if checked then
									ArkInventory.db.char.option.ldb.mounts.track = nil
								else
									ArkInventory.db.char.option.ldb.mounts.track = creatureSpellID
								end
								ArkInventory.LDB.Mounts:Update( )
							end
						)
						
						ArkInventory.Lib.DewDrop:AddLine( )
						
					end
					
					ArkInventory.Lib.DewDrop:AddLine(
						"text", ArkInventory.Localise["GENERIC_CLOSE_MENU"],
						"closeWhenClicked", true
					)
					
				end
				
			end
			
		)
	
	end
	
end
